import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;

import javax.swing.JPanel;

/**
 * Lab #8 CS 2334, Section 0?? DATE GOES HERE
 * <P>
 * This class determines the points for drawing the recursive Koch snowflake.
 * </P>
 * 
 * @version 1.0
 */

public class KochPanel extends JPanel
{
	private static final long serialVersionUID = 1L;
	private final double MIN_DIST = 25.0;
	private final double ROTATION = 60.0;
	private final double COS = Math.cos(ROTATION * Math.PI / 180.0);
	private final double SIN = Math.sin(ROTATION * Math.PI / 180.0);
	
	/*
	 * <P>
	 * This is the recursive method for the Koch curve.
	 * It contains two sections, as all recursive structures do.
	 * The base case draws a line between two points if their distance is
	 * 		less than the distance threshold 
	 * The recursive case, when the distance is more than the threshold,
	 * follows the following structure:
	 * 		F+F--F+F where
	 * 			F : paint a line
	 * 			+ : turn to the left
	 * 			- : turn to the right
	 * </P>
	 * @param		begin		The starting point of the line
	 * @param		end			The final point of the line
	 * @param		g			The Graphics object used to draw
	 */
	private void paintLine(Point begin, Point end, Graphics g)
	{
		double distance = begin.distance(end);
		
		// Check for base case
		if (true)
		{
			// Use the graphics object to draw a line from begin to end points
		}
		else // recursive case
		{
			double[] slope = slope(begin, end);
			distance /= 3.0;
			
			Point p1 = nextPoint(begin, distance, slope);

			// F
			paintLine(begin, p1, g);
			
			/*
			 * Complete the implementation of:
			 * 		+F--F+F
			 */
		}
	}
	
	/*
	 * @return		the slope between p1 and p2
	 */
	private double[] slope(Point p1, Point p2)
	{
		double deltaY = p2.getY() - p1.getY();
		double deltaX = p2.getX() - p1.getX();
		return new double[]{deltaX, deltaY};
	}
	
	/*
	 * @param		p1			The starting point
	 * @param		distance	Number of units away the next Point should be
	 * @param		slope		The slope between p1 and the returned point
	 * </P>
	 * @return		a new point in the direction of <code>slope</code> at
	 * 				<code>distance</code> units from <code>p1</code> 
	 */
	private Point nextPoint(Point p1, double distance, double[] slope)
	{
		Point nextPoint = new Point(p1);
		double c = new Point().distance(new Point((int)slope[0], (int)slope[1]));
		distance /= c;
		int deltaX = (int)(slope[0] * distance);
		int deltaY = (int)(slope[1] * distance);
		nextPoint.translate(deltaX, deltaY);
		return nextPoint;
	}
	
	/*
	 * @param		slope		The current slope being worked with
	 * </P>
	 * @return		a new array representing the slope rotated 60 degrees
	 * 				counterclockwise
	 */
	private double[] rotateLeft(double[] slope)
	{
		// return slope rotated counterclockwise ROTATION degrees
		double newX = slope[0] * COS + slope[1] * SIN;
		double newY = -1 * slope[0] * SIN + slope[1] * COS;
		return new double[]{newX, newY};
	}
	
	/*
	 * @param		slope		The current slope being worked with
	 * </P>
	 * @return		a new array representing the slope rotated 60 degrees
	 * 				clockwise
	 */
	private double[] rotateRight(double[] slope)
	{
		double newX = slope[0] * COS - slope[1] * SIN;
		double newY = slope[0] * SIN + slope[1] * COS;
		return new double[]{newX, newY};
	}
	
	/*
	 * Determines the three points on the outer triangle, then calls the
	 * recursive paintLine(Point, Point, Graphics) method to draw
	 * the full Koch snowflake
	 * </P>
	 * @param	  g			The graphics object used to draw images
	 */
	protected void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		
		// Create initial points
		Point p1 = new Point(80, 130);
		Point p2 = new Point(323, 130);
		
		double[] slope = slope(p1, p2);
		
		// Draw the Koch triangle: F--F--F
		
		// F
		paintLine(p1, p2, g);
		
		// --
		slope = rotateRight(rotateRight(slope));
		
		Point p3 = nextPoint(p2, p1.distance(p2), slope);
		
		// F
		paintLine(p2, p3, g);
		
		// --F (-- isn't actually calculated, since the final point is already known)
		paintLine(p3, p1, g);
	}
	
	// Sets the desired window size
	public Dimension getPreferredSize()
	{
		return new Dimension(400, 400);
	}
}